##MAME Library Builder v2.0#
##Dracrius - Aug 20th 2019##

Write-Output "Welcome to MAME Library Builder"
Write-Output "`n"
Write-Output "Searching for MLB Settings INI's..."

$ini_list = Get-ChildItem MLB*Settings.ini
				
if (!$ini_list){ 
	Write-Output "`n"
	Write-Output "No MLB *Name* Settings.ini's Located."
	Write-Output "The Script will now automatically close in 30s!"
	Start-Sleep -s 30
	}
else{
		Write-Output $ini_list	
		Write-Output "`n"
	}

foreach ($ini_path in $ini_list) {

		function Get-IniContent ($filePath) {
		
			$ini = @{}
			switch -regex -file $FilePath {
				"^(\s+)?;|^\s*$" # Comment or blank/whitespace line
				{
					continue
				}
				“^\[(.+)\]” # Section
				{
					$section = $matches[1]
					$ini[$section] = @{}
					$CommentCount = 0
					$listlineCount = 0
					continue
				}
				“(.+?)\s*=\s*(.*)” # Key
				{
					$name,$value = $matches[1..2]
					$ini[$section][$name] = $value
					continue
				}
				Default
				{
					$value = $_
					$listlineCount = $listlineCount + 1
					$name = "ListLine" + $listlineCount.tostring("000000")
					$ini[$section][$name] = $value
				}
			}
			return $ini
		}

	$ini=Get-IniContent $ini_path
	$root_folder = $ini['Settings']['MAME Root']
	if (!$root_folder){ 
		Write-Output "`n"
		Write-Output "No MAME Root Setting in $ini!"
		Write-Output "The Script will now automatically close in 30s!"
		Start-Sleep -s 30
	}
	$destination_name = $ini['Settings']['Library Name']
	if (!$destination_name){ 
		Write-Output "`n"
		Write-Output "No Library Name Setting in $ini!"
		Write-Output "The Script will now automatically close in 30s!"
		Start-Sleep -s 30
	}
	
	$destination_folder = "$destination_name $root_folder"
	
	Write-Output "Beginning to build $destination_folder Library"
	Write-Output "`n"
	Write-Output "Getting INI Settings..."	
	
	$folder_list = $ini['Folders to Copy'].GetEnumerator()|sort -Property name|select value
	if (!$folder_list){ 
		Write-Output "`n"
		Write-Output "Folders to Copy section is missing from $ini!"
		Write-Output "The Script will now automatically close in 30s!"
		Start-Sleep -s 30
	}
	$scrape_list = $ini['Folders to Scrape'].GetEnumerator()|sort -Property name|%{$_.value}
	if (!$scrape_list){ 
		Write-Output "`n"
		Write-Output "Folders to Scrape section is missing from $ini!"
		Write-Output "The Script will now automatically close in 30s!"
		Start-Sleep -s 30
	}
	$rom_list = $ini['ROMs List'].GetEnumerator()|sort -Property name|select value
	if (!$rom_list){ 
		Write-Output "`n"
		Write-Output "ROMs List section is missing from $ini!"
		Write-Output "The Script will now automatically close in 30s!"
		Start-Sleep -s 30
	}
	
	Write-Output "INI Settings Loaded Successfully!"
	Write-Output "`n"
	Write-Output "Beginning with Folders to Copy..."
	
	foreach ($folder in $folder_list) {
        Copy-Item $root_folder\$folder -Destination $destination_folder\$folder -Recurse
	}	
	
	Write-Output "Folder Copying Complete!"
	Write-Output "`n"
	Write-Output "Beginning Scraping..."
	
	foreach ($scrape in $scrape_list) {
		New-Item -Type dir "$destination_folder\$scrape"
		
		$scrape_folder = "$root_folder\$scrape"
		$scap_destination_folder = "$destination_folder\$scrape"
		
		Write-Output "Scraping  $scrape for Files Matching the ROMs List..."
		
		foreach ($rom in $rom_list) {
			$file_to_copy = Get-ChildItem -Path $scrape_folder -Filter $rom -Recurse -ErrorAction SilentlyContinue -Force | % { $_.FullName}
			if ($file_to_copy) {
				Copy-Item $file_to_copy $scrape_destination_folder
				}
		}
		
		Write-Output "$scrape Complete!"
	}
	
	Write-Output "`n"
	Write-Output "All Scraping  Complete!"
	Write-Output "`n"
	Write-Output "$destination_folder Library Creation Complete!"
	Write-Output "`n"
		
}

Write-Output "All SubLibrary Creation Complete!"
Write-Output "MAME Library Builder will automatically close in 10s..."
Start-Sleep -s 10